/* YourDuino Temperature and Humidity Monitor Example
 Can be used for Air Conditioner testing and other purposes.
 Reads multiple temperatures from three DS18B20 sensors
 Reads room air temperature and humidity with DHT11 sensor
 Displays on 4-line LCD 
 
 DS18B20 Chip Pinout (Left to Right, pins down, flat side toward you)
 - Left   = Ground
 - Center = Signal (Pin 10):  (with 3.3K to 4.7K resistor to +5 or 3.3 )
 - Right  = +5 or +3.3 V
 
 Waterproof Temperature DS18B20 Sensor: 
 - Black = ground
 - Red   = +5 or +3.3V
 - White = Signal pin (pin 10:  (with 3.3K to 4.7K resistor to +5 or 3.3 )
 
 DHT11 Temp/Humidity Sensor:(Left to Right, 4 pins down, open side toward you)
 - Left (1) = +5V
 - Next (2) = Signal (10K pullup to +5V)
 - Next (3) = NO CONNECTION
 - Right(4) = Ground
 
 NOTE: DHT11 responses can be seen on the Serial Monitor (Upper right IDE)
 
 Questions: terry@yourduino.com */

/*-----------------( Import needed libraries )---------------------------*/
// Get 1-wire Library here: http://www.pjrc.com/teensy/td_libs_OneWire.html
#include <OneWire.h>

//Get DallasTemperature Library here:  http://milesburton.com/Main_Page?title=Dallas_Temperature_Control_Library
#include <DallasTemperature.h>

// Get DHT11 Library here: 
#include <dht11.h>


// Get LCD Library here: https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads
// Move original LiquidCrystal library elsewhere, copy this in it's place
#include <LCD.h>
#include <LiquidCrystal_I2C.h>  // F Malpartida's NewLiquidCrystal library

// Wire (I2C) Library (Part of Arduino IDE package)
#include <Wire.h>

/*--------------( Declare Constants and Pin Numbers )-------------------*/

// DS18B20 sensors connected to port 10 on the Arduino (can be changed)
#define ONE_WIRE_BUS_PIN 10   // NOTE: No ";" on #define  
#define DHT11_PIN         3   // The Temperature/Humidity sensor

#define I2C_ADDR    0x20  // Define I2C Address for the PCF8574A on LCD Display 
//---(Following are the PCF8574 pin assignments to LCD connections )----
// This are different than earlier/different I2C LCD displays
#define BACKLIGHT_PIN  7
#define En_pin  4
#define Rw_pin  5
#define Rs_pin  6
#define D4_pin  0
#define D5_pin  1
#define D6_pin  2
#define D7_pin  3

#define  LED_OFF  0
#define  LED_ON  1


/*-----( Declare Objects )-----*/
// Setup a oneWire instance to communicate with any OneWire devices 
// (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_BUS_PIN);
// Pass address of our oneWire instance to Dallas Temperature. 
DallasTemperature sensors(&oneWire);

// Start the LCD display library
LiquidCrystal_I2C  lcd(I2C_ADDR,En_pin,Rw_pin,Rs_pin,D4_pin,D5_pin,D6_pin,D7_pin);

// Start the DHT11 Temperature/Humidity Sensor library
dht11 DHT11;


/*----------------------( Declare Variables )-----------------------------*/
// Assign the addresses of your 1-Wire temp sensors.
// See the tutorial on how to obtain these addresses:
// http://www.hacktronics.com/Tutorials/arduino-1-wire-address-finder.html

DeviceAddress Probe01 = { 0x28, 0x9A, 0x80, 0x40, 0x04, 0x00, 0x00, 0xD5 }; // "4"
DeviceAddress Probe02 = { 0x28, 0xE1, 0xC7, 0x40, 0x04, 0x00, 0x00, 0x0D }; // "5"
DeviceAddress Probe03 = { 0x28, 0x9A, 0x80, 0x40, 0x04, 0x00, 0x00, 0xD5 }; // "4" Again for test


void setup()   /************* SETUP: RUNS ONCE ***************/
{
  Serial.begin(9600); //(Remove all 'Serial' commands if not needed)   

  //------- Initialize the DS18B20 Temperature measurement library--------------
  sensors.begin();
  // set the resolution to 10 bit (Can be 9 to 12 bits .. lower is faster)
  sensors.setResolution(Probe01, 10);
  sensors.setResolution(Probe02, 10);
  sensors.setResolution(Probe03, 10);
 

  //---------------- Initialize the lcd ------------------  
  lcd.begin (20,4);  // 20 characters, 4 lines
  // Switch on the backlight
  lcd.setBacklightPin(BACKLIGHT_PIN,NEGATIVE);
  lcd.setBacklight(LED_ON); 
  
  //---------------- Display welcome message ------------------
  lcd.setCursor(0,0); //Start at character 0 on line 0
  lcd.print("DS18B20 Temperatures");
  lcd.setCursor(0,1); //Start at character 0 on line 1
  lcd.print("DHT11 Temp+Humidity");  
  lcd.setCursor(0,2); //Start at character 0 on line 2
  lcd.print("Terry@YourDuino.com");  
  delay(3000);

}//--(end setup )---


void loop()   /*************** LOOP: RUNS CONSTANTLY ****************/
{
  sensors.requestTemperatures(); // Get DS18B20 temperatures
  
  //--( Read DHT11 and check response, display errors on serial port )----  
  int chk = DHT11.read(DHT11_PIN);  
  Serial.print("Read DHT11 sensor: ");
  switch (chk)
  {
    case 0: Serial.println("OK"); break;
    case -1: Serial.println("Checksum error"); break;
    case -2: Serial.println("Time out error"); break;
    default: Serial.println("Unknown error"); break;
  }  
  
  lcd.clear();  // Reset the display  
  lcd.home();
  lcd.backlight();  //Backlight ON if under program control  
  
// Print our characters on the LCD
// NOTE: Line number and character number start at 0 not 1
  
  lcd.setCursor(0,0); //Start at character 0 on line 0
  lcd.print("S1: ");
  displayTemperature(Probe01);  
  
  lcd.setCursor(0,1); //Start at character 0 on line 1
  lcd.print("S2: ");
  displayTemperature(Probe02);  
  
  lcd.setCursor(0,2); //Start at character 0 on line 2
  lcd.print("S3: ");
  displayTemperature(Probe03);  
  // Display DHT11 Temperature and Humidity 

  lcd.setCursor(0, 3); //Start at character 0 on line 3

  lcd.print("ROOM C=");
  lcd.print((float)DHT11.temperature, 0);
  Serial.println("ROOM TEMPERATURE AND HUMIDITY: ");  
  Serial.print("Temperature (C): ");
  Serial.println((float)DHT11.temperature, 2);

  lcd.print(" F=");
  lcd.print(Fahrenheit(DHT11.temperature), 0);
  Serial.print("Temperature (F): ");
  Serial.println(Fahrenheit(DHT11.temperature), 2);
  
  lcd.print(" H=");
  lcd.print((float)DHT11.humidity, 0);
  lcd.print("%");
  Serial.print("Humidity    (%): ");
  Serial.println((float)DHT11.humidity, 2);
  Serial.println();  
 
  delay(2000);

}//--(end main loop )---

/*-----( Declare User-written Functions )-----*/

void displayTemperature(DeviceAddress deviceAddress)
{

float tempC = sensors.getTempC(deviceAddress);

   if (tempC == -127.00) // Measurement failed or no device found
   {
    lcd.print("Temperature Error");
   } 
   else
   {
   lcd.print("C=");
   lcd.print(tempC);
   lcd.print(" F=");
   lcd.print(DallasTemperature::toFahrenheit(tempC)); // Convert to F
   }
}// End displayTemperature

  //-----------------------------------------------------------------
//Celsius to Fahrenheit conversion
double Fahrenheit(double celsius)
{
        return 1.8 * celsius + 32;
}
  //---------------------------------------------------------------
//Celsius to Kelvin conversion
double Kelvin(double celsius)
{
        return celsius + 273.15;
}  
  //---------------------------------------------------------------
// reference: http://en.wikipedia.org/wiki/Dew_point
double dewPointFast(double celsius, double humidity)
{
        double a = 17.271;
        double b = 237.7;
        double temp = (a * celsius) / (b + celsius) + log(humidity/100);
        double Td = (b * temp) / (a - temp);
        return Td;
}


//*********( THE END )***********


