 /*
//==========================================================================//
 //                                                                          //
 //   AI-1 ("All-in-one") Remote      Version 1.02     2  Dec 2010           // 
 //   KAUST Photography Group                                                //
 //   terry@terryking.us                                                     //
 //                                                                          //
 //   Written for the Arduino ATmega168 installed in the Jul 18,2007         //
 //   version of the AI-1 Remote hardware.                                   //
 //   http://www.mindspring.com/~tom2000                                     //
 //==========================================================================//
 
 Copyright 2007, Tom Lackamp
 Released into the public domain under the provisions of the
 GNU General Public License
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 For a copy of the GNU General Public License, refer to 
 http://www.gnu.org/licenses/
 
 //==========================================================================//        
 
 Program compiles to 14054 bytes of 14336 bytes available, 
 using the Arduino-0008 development system          
 
 //==========================================================================//
 //                                                                          //
 //                            Contents                                      //
 //                                                                          //
 //==========================================================================//
 
 Listing Contents:
 
 Preamble:
 
 Supercap configuration definition
 delete or comment out if you back up your DS1302 with a battery.
 
 System Tuning Constants
 
 I/O Pin Assignments
 
 DS1302 Defines
 
 Global RAM Declarations
 
 EEPROM Alias Defines
 
 Constant Byte Data
 
 Constant LCD Strings
 
 System Mode Defines
 
 
 Code Body:
 
 Note:  after booting, the Arduino executes the 
 setup() function, then the loop() function.
 
 setup()
 
 loop()
 
 Dispatcher  (CheckSecs())
 
 Power Bus Monitor (CheckPwrBus() and Backlight routines)
 
 System Initialization routines
 
 Common Button routines - functions used by the 
 Menu, Select, Up, and Down buttons
 
 Menu Button routines - doMenuBtn and its support functions
 
 The doMenuBtn routine is the heart of the system.  
 Learning, understanding, and maintaining the program begins
 in this routine.
 
 Menu Button Screen routines
 
 Menu Button Support routines
 
 Select Button routines - doSelBtn and its support functions
 
 Common Up/Down Button routines - support functions used by 
 both the Up and Down button routines
 
 Up Button routines - doUpBtn and its support functions
 
 Down Button routines - doDownBtn and its support functions
 
 Service routines - misc functions
 
 8-byte Time routines - functions used to support the 
 "thousand hour" time format
 
 Shoot and External Trigger routines  - exposure mode dispatcher 
 and low-level exposure routines
 
 Alarm Clock routines
 
 Intervalometer Routines      
 
 LCD Functions - LCD stuff like clear, backspace, icon display, etc.
 
 LCD Driver routines - low-level code to provide the 4-bit parallel 
 LCD interface
 
 DS1302 Driver routines - low-level pseudo-SPI interface to the 
 DS1302 real time clock
 
 Debug routines - breakpoint and display-to-LCD debugging 
 routines. 
 
 //==========================================================================//
 //                                                                          //
 //                            Preamble                                      //
 //                                                                          //
 //==========================================================================//
 */

#include <EEPROM.h> 

//  If you use a battery backup for your clock, delete or comment out 
//  the following line -- "#define Supercap" -- to disable the DS1302's 
//  trickle charger

#define Supercap

//  System tuning constants:  change these, if you wish, for your preferences.

#define BLHangtime        10      //  Seconds (approx) backlight stays on 
//  after keypress while on battery
#define AutoRepeatThresh  25     //  Number of 10 mSec debounce periods 
//  before autorepeat for Up and Down buttons
#define LowBattThresh     233    //  ADC value for 6.5 V unreg 
//  power bus measurement
#define ExtPwrThresh      359    //  ADC value for 10 V unreg 
//  power bus measurement
//  (values based on tap from 
//  47k + 10k voltage divider on power bus) 
#define SnoozeTime        10     //  Snooze time minutes                                          
#define BeeperTimeout     15     //  Minutes until the beeper shuts itself off 
#define SetupTimeout      2      //  Minutes before setup mode times out to idle  
#define SplashScrnTime    5      //  Splash Screen display time, seconds  


// Input connections

int MenuBtn   = 0;              
int SelBtn    = 1;
int UpBtn     = 4;
int DownBtn   = 5;
int PwrBus    = 5;               //  An5 monitors power bus
int ShootSw   = 2;
int ExtTrig   = 3;

//  DS1302 connections

int DAT1302   = 7;
int CLK1302   = 6;
int RST1302   = 8;

// Output connections

int BLpin     = 9;                //  Backlight
int FocusPin  = 12;
int ShootPin  = 13;
int AlarmPin  = 11;
int DB4       = 14;
int DB5       = 15;
int DB6       = 16;
int DB7       = 17;
int RS        = 18;
int E         = 10;   

//  DS1302 Opcodes

#define WriteCtrl     B10001110
#define ReadSecs      B10000001
#define WriteSecs     B10000000
#define ReadMins      B10000011
#define WriteMins     B10000010
#define ReadHrs       B10000101
#define WriteHrs      B10000100
#define WriteTrickle  B10010000
#define ReadDay       B10001011
#define WriteDay      B10001010
#define ReadDate      B10000111
#define WriteDate     B10000110
#define ReadMonth     B10001001
#define WriteMonth    B10001000
#define ReadYear      B10001101
#define WriteYear     B10001100

#ifdef Supercap

//  Sets DS1302 trickle charger for supercap backup
//  1 diode, 2k series resistance

#define TrickleSet    B10100101	

#else

//  Sets DS1302 for no trickle charger output.  Use this for battery backup.

#define TrickleSet     B11111111

#endif

//  System globals

byte Secs;        //  DS1302's BCD values  
byte Mins;
byte Hrs;         //  Hrs has 12/24 and AM/PM bits stripped
byte PMFlag;
byte Mode;
byte ExpMode;
byte IntvlMode;
byte DigPtr;
byte MaxDig;
byte MaxVal;
byte MinVal;
byte YesNo; 
byte DispBase;
byte UpDownWork;
byte BLWork;
byte BLCounter;        //  Times backlight hang time while on battery
byte IntvlSetRegs[9];  //  Tenths, Sx1, Sx10, Mx1, Mx10, 
//  Hx1, Hx10, Hx100, dummy
long IntvlSetTenths;
byte IntvlRunRegs[9];
long IntvlRunTenths;
byte TimexSetRegs[9];
long TimexSetTenths;
byte TimexRunRegs[9];
long TimexRunTenths;
byte RunTimeRegs[9];
long RunTimeTenths;
int  Shots;
byte ShotRegs[4];
byte AlmTime[4];       //  AlmFlag, AlmHrs, AlmMins, AlmPM flag.  
//  Hrs and Mins in BCD.  
//  Flags = 1 (set / PM) or 0 (not set / AM). 
byte AlmScpd[3];       //  Alarm Time scratchpad:  bcd Hrs, Mins, PMflag   
byte BeepTimeout[3];   //  Beep timeout time                      
byte StartTimeRegs[3];
byte StopTimeRegs[3];
byte DateRegs[4];      //  Day (1-7, Sun = 1), Month, Date, Year (BCD)     
int  ShotsTaken;  
byte IdleCounter;
byte DayMap;

boolean Smudged;
boolean BLstate;
boolean ExtPwrFlag;
boolean BattState;
boolean LowBattFlag;
boolean IntvlWait;
boolean AlmSounding;
boolean SnoozeFlag;
boolean DonePending;

//  EEPROM locations

#define InitEEPROM     0x0
#define FocusSecs      0x1
#define ShootSecs      0x2
#define BLBright       0x3
#define ExpModeLoc     0x4
#define IntvlModeLoc   0x5
#define IntvlFrac      0x6
#define IntvlSecx1     0x7
#define IntvlSecx10    0x8
#define IntvlMinx1     0x9
#define IntvlMinx10    0xa
#define IntvlHrx1      0xb
#define IntvlHrx10     0xc
#define IntvlHrx100    0xd
#define TimexFrac      0xe
#define TimexSecx1     0xf
#define TimexSecx10    0x10
#define TimexMinx1     0x11
#define TimexMinx10    0x12
#define TimexHrx1      0x13
#define TimexHrx10     0x14
#define TimexHrx100    0x15  
#define RunTimeFrac    0x16
#define RunTimeSecx1   0x17
#define RunTimeSecx10  0x18
#define RunTimeMinx1   0x19
#define RunTimeMinx10  0x1a
#define RunTimeHrx1    0x1b
#define RunTimeHrx10   0x1c
#define RunTimeHrx100  0x1d 
#define ShotsHi        0x1e
#define ShotsLo        0x1f
#define AlarmSetLoc    0x20
#define AlarmHrs       0x21
#define AlarmMins      0x22
#define AlarmPM        0x23
#define StartTimeHrs   0x24
#define StartTimeMins  0x25
#define StartTimePM    0x26
#define StopTimeHrs    0x27
#define StopTimeMins   0x28
#define StopTimePM     0x29
#define ShotsTakenHi   0x2a
#define ShotsTakenLo   0x2b
#define DayMapLoc      0x2c
#define IntvlWaitLoc   0x2d


//  LCD init data:  4-bit mode, 2-line display, no display shift,
//  cursor off, 5x7 font
const byte IData[] = {
  0x33,0x32,0x28,0x0c,0x06};  

//  Max values for each digit in the 8-byte time registers
//  used by DecTimeRegs()
const byte TimeMax[] = {
  9,9,5,9,5,9,9,9};           

//  Low Battery icon LCD char RAM data - "AA Cell"
const byte LBIcon[] = {
  B00000110,B00001111,B00001001,B00001001,
  B00001001,B00001001,B00001111,B00000000}; 

//  Alarm Set icon LCD char RAM data - "Bell"
const byte AlmIcon[] = {
  B00000000,B00000100,B00001010,B00001010,
  B00011111,B00000100,B00000000,B00000000}; 

//  Intervalometer Timed-start icon LCD char RAM data - "Hourglass"                          
const byte IntvlIcon[] = {
  B00011111,B00001010,B00001010,B00000100,
  B00001010,B00001010,B00011111,B00000000}; 

//  Backlight brightness values
const byte BLVals[] = {
  0,31,63,127,255}; 

// Max days for each month
const byte MaxDays[13] = {
  99,31,29,31,30,31,30,31,31,30,31,30,31};                           

//  LCD Screen text                        
const char Title[] =   {
  "CameraController"};
const char Version[] = {
  "PHOTO GROUP 1.02"};
const char StartAutoStr[] = {
  "Start Auto Run"};
const char StopAutoStr[] = {
  "Stop Auto Run"};
const char YesStr[] = {
  "Yes"};
const char NoStr[] = {
  "No "}; 
const char FocusStr[] = {
  "Set Focus Time"};
const char ShootStr[] = {
  "Set Shoot Time"};
const char TimeExStr[] = {
  "Time Exp Set"};
const char ClearStr[] = {
  "Clr Auto Set?"};
const char SetTimeStr[] = {
  "Set Time"};
const char SetDateStr[] = {
  "Set Date"};
const char AlarmStr[] = {
  "Alarm O"};
const char SetAlarmStr[] = {
  "Set Alarm Time"};
const char SetIntvlStr[] = {
  "Set Interval"};
const char ShotsTakenStr[] = {
  " Shots Tkn"};
const char ClearTokenStr[] = {
  "Clear? "};
const char MaxShotStr[] = {
  "Set Max Shots"};
const char RunTimeStr[] = {
  "Set Run Time"};
const char StartTimeStr[] = {
  "Set Start Time"};
const char StopTimeStr[] = {
  "Set Stop Time"};
const char BLStr[] = {
  "BL Brightness"};
const char DoneStr[] = {
  "Done"};
const char ChooseStr[] = {
  "Set Exp Mode"};
const char NormStr[] = {
  "Norm"};
const char BulbStr[] = {
  "Bulb"};
const char TimeExpStr[] = {
  "Time"};
const char AutoModeStr[] = {
  "Set Auto Mode"};
const char IntvlShotStr[] = {
  "Shot Limited"};
const char IntvlTimeLimStr[] = {
  "Time Limited"};
const char IntvlTimedStr[] = {
  "Timed"};
const char PwrBusStr[] = {
  "Bus Voltage"};
const char VDCStr[] = {
  " VDC"};
const char IdleStr[] = {
  "Idle"};
const char AuxModeStr[] = {
  "Alarm Output"};
const char BeepStr[] = {
  "Beeping"};
const char SteadyStr[] = {
  "Steady "};
const char BulbModeErrStr1[] = {
  "You can't use"};
const char BulbModeErrStr2[] = {
  "Bulb in Auto"};
const char DurErrStr1[] = {
  "Run Time is"};
const char DurErrStr2[] = {
  "set to zero"};
const char MaxShotsErrStr1[] = {
  "Max Shots"};
const char MaxShotsErrStr2[] = {
  "must be > 0"};
const char ShotsTakenErrStr[] = {
  "Shots Taken >"};     
const char DayMapStr[]= {
  "Set Run Days"};
const char DOWStr[8][4] = {
  "   ","Sun","Mon","Tue",
  "Wed","Thu","Fri","Sat"};
const char MonthStr[13][4] = {
  "   ","Jan","Feb","Mar","Apr","May","Jun",
  "Jul","Aug","Sep","Oct","Nov","Dec"};

//  Modes of Operation

#define Idle          0
#define ClockSet      1
#define Alarm         2
#define AlarmSet      3
#define StartPending  4
#define StopPending   5
#define Focus         6
#define Shoot         7
#define SetExpMode    8
#define SetTimeExp    9
#define ClrAuto       10
#define SetInterval   11
#define ClearShots    12
#define SetAutoMode   13
#define SetShots      14
#define SetRunTime    15
#define SetStartTime  16
#define SetStopTime   17
#define BLSet         18
#define BusVolts      19
#define RunMode	      20
#define DoneMode      21
#define ErrorMode     22
#define DayMapSetMode 23
#define StopIntvlWait 24
#define DateSetMode   25

//  Exposure modes

#define ExpNormal     0
#define ExpBulb       1
#define ExpTimeEx     2

//  Intervalometer modes

#define ShotLimited   0
#define TimeLimited   1
#define Timed         2

//==========================================================================//
//                                                                          //
//                            Code Body                                     //
//                                                                          //
//==========================================================================//


// #define Debug

void setup()
{
  delay(250);                      //  Let everything wake up
  InitSystem();
}

void loop()
{
  byte ms,ls;

  while(1)
    TopScan();
}

void TopScan()
{

  if ((PIND & B00000100) || !(PIND & B00001000))    //  ShootSw high or 
    TakeAPicture();                                 //  ExtTrig low
  if (!(PIND & B00000001))                          //  MenuBtn low
    doMenuBtn();
  if (!(PIND & B00000010))                          //  SelBtn low
    doSelBtn();
  if (!(PIND & B00010000))                          //  UpBtn low
    doUpBtn();
  if (!(PIND & B00100000))                          //  DownBtn low
    doDownBtn();
  CheckPwrBus();

  if ((Mode == Idle) || AlmSounding || (Mode == RunMode))
    CheckSecs();
  else if ((Mode != DoneMode) && (Mode != ErrorMode))
    IdleTimeoutTest();
} 

void IdleTimeoutTest()
{
  //  Provides a 2-minute setup mode timeout before return to Idle mode.

  static byte PrevSecs;

  if (!IdleCounter)
    return;

  Secs = get1302data(ReadSecs);
  if (Secs != PrevSecs)
  {
    PrevSecs = Secs;
    IdleCounter-- ;
    if (!IdleCounter)
      IdleModeReset();
  }
}  

//============================== Dispatcher =================================//

void CheckSecs()
{
  byte x=0;
  byte ms,ls;
  static byte PrevSecs;

  //  This routine is the 1-second event dispatcher

  Secs = get1302data(ReadSecs);
  if (Secs != PrevSecs)        //  1 Second Dispatcher
  {
    PrevSecs = Secs;
    GetTime();    

#ifndef Debug

    ShowTime(0x82);

#endif

    if (!ExtPwrFlag && BattState && BLCounter > 0 && !LowBattFlag)
    {
      BLCounter-- ;
      if (!BLCounter)
        BacklightOff();
    }

    if (AlmSounding)
      Beep();

    if (SnoozeFlag && AlmTime[0])      //  flash alarm icon
      ((Secs % 2) == 0) ? ClearAlmIcon() : ShowAlmIcon() ;      

    if (Mode == RunMode)
    {
      if (IntvlRunTenths == 0)
        ResetIntvl();
      IntvlRunTenths-- ;
      DisplayCountdown();
    }

    if ((Mode == RunMode) && (IntvlMode == TimeLimited))
    {
      RunTimeTenths-- ;
      if (RunTimeTenths == 0)
        SetDoneMode();
    }

    if (Secs == 0)            //  1 Minute Dispatcher
    {
      if (AlmSounding && (Hrs == BeepTimeout[0]) &&
        (Mins == BeepTimeout[1]) && (PMFlag == BeepTimeout[2]))
      {
        //  Beep timeout.  Shut the alarm down
        AlmSounding = 0;
        SnoozeFlag = 0;
        IdleModeReset();  //  Return to the idle-mode display
      }

      if (AlmTime[0] && (Hrs == AlmScpd[0]) && 
        (Mins == AlmScpd[1]) && (PMFlag == AlmScpd[2]))
      {
        //  Trigger the alarm
        AlmSounding = 1;
        //  Calculate the time the beeper will shut itself off
        BeepTimeout[0] = Hrs;
        BeepTimeout[1] = Mins;
        BeepTimeout[2] = PMFlag;
        Inc4DigBCDTime(BeepTimeout,BeeperTimeout); 
      }

      if (IntvlMode == Timed)
      {
        IntvlWait = EEPROM.read(IntvlWaitLoc);
        if (IntvlWait)
        {
          GetDate();
          if ((EEPROM.read(StartTimeHrs) == Hrs) &&
            (EEPROM.read(StartTimeMins) == Mins) &&
            (EEPROM.read(StartTimePM) == PMFlag) &&
            (DayMap & (1 << DateRegs[0])))
          {
            x = CheckIntvlStartErrs();
            if (x)
              return;
            IntvlWait = 0;
            EEwrite(IntvlWaitLoc,0);
            ClearL2();
            Mode = RunMode;
            TakeAPicture();
          }
        }
        else
        {
          if ((EEPROM.read(StopTimeHrs) == Hrs) &&
            (EEPROM.read(StopTimeMins) == Mins) &&
            (EEPROM.read(StopTimePM) == PMFlag))
          {
            IntvlWait = 1;
            EEwrite(IntvlWaitLoc,1);
            ShowIcons();
            IdleModeReset();
          }
        }
      } 
    }
  }
}

//=====================  Power Bus Monitor ==================//

void CheckPwrBus()
{
  int Volts;
  static boolean PrevExtPwrFlag;

  Volts = analogRead(PwrBus);
  if (Volts > ExtPwrThresh)
  {
    if (LowBattFlag)
    {
      LowBattFlag = 0;
      ClearLBIcon();
    }
    ExtPwrFlag = 1;
    PrevExtPwrFlag = 1;
    BattState = 1;
    LowBattFlag = 0;
    BacklightOn();
  }
  else if (Volts > LowBattThresh)
  {
    if (LowBattFlag)
    {
      LowBattFlag = 0;
      ClearLBIcon();
    }
    ExtPwrFlag = 0;
    BattState = 1;
    LowBattFlag = 0;
    if (PrevExtPwrFlag)
    {
      BLCounter = BLHangtime;
      PrevExtPwrFlag = 0;
    }
  }
  else
  {
    ExtPwrFlag = 0;
    BattState = 0;
    if (!LowBattFlag)
    {
      ShowLBIcon();
      LowBattFlag = 1;
    }
    if (BLstate)
      BacklightOff();
  } 
}

void BacklightOn()
{
  //  if (!BLstate && BattState  && !LowBattFlag)
  if (!BLstate && !LowBattFlag)
  {
    analogWrite(BLpin,BLVals[EEPROM.read(BLBright)]);
    BLstate = 1;
  }
}

void BacklightOff()
{
  analogWrite(BLpin,0);
  BLstate = 0;
}      

//==============  System Initialization Routines =================//

void InitSystem()
{
  //  Config and initialize I/O lines

  int i;

  DDRD = B11000000;
  PORTD = B00110011;     // pullups on Menu, Sel, Up, and Down buttons
  DDRB |= B00111111;
  PORTB &= B11000000;
  DDRC |= B00011111;
  PORTC &= B11011111;

  Init1302();
  InitLCD();
  AlmSounding = 0;
  SnoozeFlag = 0;
  LowBattFlag = 1;          // to kickstart power bus monitor process
  CheckPwrBus();
  if (!LowBattFlag)
    analogWrite(BLpin,BLVals[EEPROM.read(BLBright)]);
  SignonMsg();
  delay(SplashScrnTime * 1000);
  BLCounter = BLHangtime;
  Clear();
  GetTime();
  ShowTime(0x82);

  // Load default values to EEPROM if this is a fresh program load
  if (EEPROM.read(InitEEPROM) != 0x5a)
  {
    EEwrite(FocusSecs,5);
    EEwrite(ShootSecs,5);
    EEwrite(BLBright,3);
    EEwrite(ExpModeLoc,ExpNormal);
    EEwrite(IntvlModeLoc,ShotLimited);
    EEwrite(IntvlFrac,0);
    EEwrite(IntvlSecx1,0);
    EEwrite(IntvlSecx10,3);
    EEwrite(IntvlMinx1,0);
    EEwrite(IntvlMinx10,0);
    EEwrite(IntvlHrx1,0);
    EEwrite(IntvlHrx10,0);
    EEwrite(IntvlHrx100,0);
    EEwrite(TimexFrac,5);
    EEwrite(TimexSecx1,1);
    EEwrite(TimexSecx10,0);
    EEwrite(TimexMinx1,0);
    EEwrite(TimexMinx10,0);
    EEwrite(TimexHrx1,0);
    EEwrite(TimexHrx10,0);
    EEwrite(TimexHrx100,0);
    EEwrite(RunTimeFrac,0);
    EEwrite(RunTimeSecx1,0);
    EEwrite(RunTimeSecx10,0);
    EEwrite(RunTimeMinx1,0);
    EEwrite(RunTimeMinx10,0);
    EEwrite(RunTimeHrx1,3);
    EEwrite(RunTimeHrx10,0);
    EEwrite(RunTimeHrx100,0);
    EEwrite(ShotsHi,0x01);      // 425 shots
    EEwrite(ShotsLo,0xa9);
    EEwrite(AlarmSetLoc,0);
    EEwrite(AlarmHrs,0x07);
    EEwrite(AlarmMins,0x30);
    EEwrite(AlarmPM,0);
    EEwrite(StartTimeHrs,0x08);
    EEwrite(StartTimeMins,0);
    EEwrite(StartTimePM,0);
    EEwrite(StopTimeHrs,0x04);
    EEwrite(StopTimeMins,0x30);
    EEwrite(StopTimePM,1);
    EEwrite(ShotsTakenHi,0);
    EEwrite(ShotsTakenLo,0);
    EEwrite(DayMapLoc,B00111110);
    EEwrite(IntvlWaitLoc,0);
    EEwrite(InitEEPROM,0x5a);
  }
  GetAlmTime();
  ExpMode = EEPROM.read(ExpModeLoc);
  IntvlMode = EEPROM.read(IntvlModeLoc);
  IntvlWait = EEPROM.read(IntvlWaitLoc);
  DayMap = EEPROM.read(DayMapLoc);
  ShowIcons();
  Mode = Idle;  
}

void Init1302()
{
  //  This routine is, most likely, called with the DS1302 running on the 
  //  supercap or battery backup, so it is important to preserve its 
  //  time values.

  send1302cmd(WriteCtrl,0);               // Clear write protect

  Secs = get1302data(ReadSecs);           // Clear CH bit, preserving secs reg
  Secs &= B01111111;
  send1302cmd(WriteSecs,Secs);

  Hrs = get1302data(ReadHrs);              // Set 12 hour format, 
  // preserving hrs reg value
  (Hrs & B00100000) ? PMFlag = 1 : PMFlag = 0;
  Hrs |= B10000000;                        // bit 7 in 1302's hours reg 
  // sets 12 hour format
  send1302cmd(WriteHrs,Hrs);
  Hrs &= B00011111;                        //  Store Hrs locally 
  //  without flag bits

  send1302cmd(WriteTrickle,TrickleSet);    // begin charging the supercap  
}

void InitLCD()
{
  //  Sends 5 initialization bytes to LCD
  int i;

  for (i=0; i<5; i++)
  {
    Lcmd(IData[i]);
    delay(15);
  }

  Lcmd(0x40);              //  Load Alarm Set icon into CG RAM char 0
  for (i=0; i<8; i++)
    Ldat(AlmIcon[i]);
  Lcmd(0x80);

  Lcmd(0x48);              //  Load Low Battery icon into CG RAM char 1
  for (i=0; i<8; i++)
    Ldat(LBIcon[i]);
  Lcmd(0x80);

  Lcmd(0x50);              //  Load Intervalometer timed-start icon 
  //  into CG RAM char 2
  for (i=0; i<8; i++)
    Ldat(IntvlIcon[i]);
  Lcmd(0x80);
}

void SignonMsg()
{  
  Clear();
  SendStr(Title,0x80);
  SendStr(Version,0xc0);
  // ORIG   SendStr(Version,0xc2);

}

void GetAlmTime()
{
  AlmTime[0] = EEPROM.read(AlarmSetLoc);
  AlmTime[1] = EEPROM.read(AlarmHrs);
  AlmTime[2] = EEPROM.read(AlarmMins);
  AlmTime[3] = EEPROM.read(AlarmPM);

  for (int i=0; i<3; i++)
    AlmScpd[i] = AlmTime[i + 1];
}    

//===================== Common Button Routines ==============================//

void debounce(int pin)
{
  int rptcntr = 0;

  if (pin == UpBtn || pin == DownBtn)
  {
    //  Autorepeat the Up and Down buttons
    while(!digitalRead(pin) && (rptcntr++ < AutoRepeatThresh))
      delay(10);
  }
  else
  {
    while(!digitalRead(pin))
      delay(10);
  }
  delay(10);
  BLCounter = BLHangtime;
  BacklightOn();

  IdleCounter = 60 * SetupTimeout;
}

void ShowShootTime()
{
  //  Shows shoot and focus time as "n.n"
  Lcmd(0xc6);
  Ldat(UpDownWork / 10 + '0');
  Ldat('.');
  Ldat(UpDownWork % 10 + '0');
  Lcmd(0xc6);
} 

void ShowAutoMode()
{
  ClearL2();
  if (UpDownWork == ShotLimited)
    SendStr(IntvlShotStr,0xc2);
  else if (UpDownWork == TimeLimited)
    SendStr(IntvlTimeLimStr,0xc2);
  else       // Timed
  SendStr(IntvlTimedStr,0xc5);
}

void SetSnoozeAlm()
{
  AlmSounding = 0;
  SnoozeFlag = 1;
  Inc4DigBCDTime(AlmScpd,SnoozeTime);     //  Add 10 mins to Alarm Scratchpad 
  //  registers
}  

void IdleModeReset()
{
  Clear();
  GetTime();
  ShowTime(0x82);
  Mode = Idle;
  HideCurs();
}

//========================  Menu Button Routines ============================//      

void doMenuBtn()
{
  //  With the exception of the alarm clock, which is controlled by 
  //  boolean flags AlmSounding and SnoozeFlag and the alarm set flag
  //  in AlmTime[0], the operation of the entire system is determined by
  //  the Mode setting.

  //  The Menu button is the key to the system.  When pressed, doMenuBtn
  //  steers the operation via a switch statment on Mode.  

  //  A typical Mode case consists of a closeout of the previous mode's
  //  setup operation, followed by a routine that sets up the system
  //  for the subsequent mode.  The setup routine sets Mode to steer 
  //  the next Menu button press to the appropriate routine, and to
  //  determine the operation of the Select, Up, Down, and Shoot buttons.

  int i;
  byte ms,ls;

  debounce(MenuBtn);

  if (AlmSounding || SnoozeFlag)
  {
    AlmSounding = 0;      //  Shut down the alarm
    SnoozeFlag = 0;
    ShowIcons();
    if (Mode != RunMode)
      IdleModeReset();    //  Back to Idle mode
    return;
  }

  //  Note:  changing from a switch structure to an if-then-else structure
  //         saved 266 compiled bytes of program memory

  if (Mode == Idle)
    SetIntvlStartScreen(StartPending);
  else if (Mode == StartPending)
    if (YesNo)
      CloseStartPending();   // Start the intervalometer
    else
    {
      ClearL2(); 
      SetClockSetScreen(ClockSet);
    }
  else if (Mode == ClockSet)
  {
    if (Smudged)
      CloseClockSet(); 
    SetDateSetScreen(DateSetMode);
  }
  else if (Mode == DateSetMode)
  {
    if (Smudged)
      CloseDateSet();
    SetAlarmOnOffScreen(Alarm);    
  }
  else if (Mode == Alarm)
    if (AlmTime[0])
      SetAlarmTimeSet(AlarmSet);
    else
      SetFocusScreen(Focus);
  else if (Mode == AlarmSet)
  {
    if (Smudged)
      CloseAlmTimeSet();   // Write alarm time to EEPROM
    else
    {
      AlmSounding = 0;
      SnoozeFlag = 0;
    }
    SetFocusScreen(Focus);
  }
  else if (Mode == Focus)
  {
    if (Smudged)
      EEwrite(FocusSecs,UpDownWork);
    SetShootTimeScreen(Shoot);
  }
  else if (Mode == Shoot)
  {
    if (Smudged)
      EEwrite(ShootSecs,UpDownWork);
    SetExpModeScreen(SetExpMode);
  }
  else if (Mode == SetExpMode)
  {
    if (Smudged)
      EEwrite(ExpModeLoc,ExpMode);
    if (ExpMode == ExpTimeEx)
      SetTimexTimeScreen(SetTimeExp);    
    else
      SetClrAutoScreen(ClrAuto); 
  }      
  else if (Mode == SetTimeExp)
  {
    if (Smudged)
      SaveTimeRegs(TimexFrac,TimexSetRegs);
    SetClrAutoScreen(ClrAuto);
  }
  else if (Mode == ClrAuto)
  {
    if (YesNo)
      ClearAllIntervalometerSettings();
    SetIntvlTimeScreen(SetInterval);
  }
  else if (Mode == SetInterval)
  {
    if (Smudged)
      SaveTimeRegs(IntvlFrac,IntvlSetRegs);
    SetMaxShotsScreen(SetShots);
  }
  else if (Mode == SetShots)
  {
    if (Smudged)
      CloseMaxShots();
    SetClearShotsScreen(ClearShots);
  }    
  else if (Mode == ClearShots)
  {
    if (YesNo)
      CloseClearShots();
    SetIntvlModeScreen(SetAutoMode);
  }
  else if (Mode == SetAutoMode)
  {
    if (Smudged)
    {
      EEwrite(IntvlModeLoc,UpDownWork);
      IntvlMode = UpDownWork;
      if (IntvlMode == Timed)
        CheckIntvlStartErrs();
    }

    (UpDownWork == Timed) ? IntvlWait = 1 : IntvlWait = 0;
    EEwrite(IntvlWaitLoc,IntvlWait);

    Clear();
    if (UpDownWork == ShotLimited)    //  UpDownWork comes back 
      ShowBusVoltage(BusVolts);       //  containing the intervalometer auto mode
    else if (UpDownWork == TimeLimited)
      SetRunTimeScreen(SetRunTime);
    else               // Timed
    SetStartTimeScreen(SetStartTime);
  }
  else if (Mode == SetRunTime)
  {
    if (Smudged)
      SaveTimeRegs(RunTimeFrac,RunTimeRegs); 
    ShowBusVoltage(BusVolts);  
  }        
  else if (Mode == SetStartTime)
  {
    if (Smudged)
      for (i=0; i<3; i++)
        EEwrite(StartTimeHrs + i, StartTimeRegs[i]);
    SetStopTimeScreen(SetStopTime);
  }
  else if (Mode == SetStopTime)
  {
    if (Smudged)
      for (i=0; i<3; i++)
        EEwrite(StopTimeHrs + i, StopTimeRegs[i]);
    SetupDayMapScreen(DayMapSetMode);   
  }      
  else if (Mode == DayMapSetMode)
  {
    if (Smudged)
      EEwrite(DayMapLoc,DayMap);
    GetDate();
    ShowBusVoltage(BusVolts);  
  }      
  else if (Mode == BusVolts)
    SetupBLBright(BLSet);  
  else if (Mode == BLSet)
  {
    if (Smudged)
      EEwrite(BLBright,BLWork);
    IdleModeReset();
  }    
  else if (Mode == RunMode)
  {
    Clear();
    SetStopAutoScreen(StopPending);
  }
  else if (Mode == StopPending)
  {
    if (YesNo)
    {
      if (IntvlMode == Timed)
      {
        IntvlWait = 1;
        EEwrite(IntvlWait,1);
      }
      IdleModeReset();      
    }
    else
    {
      Clear();
      GetTime();
      ShowTime(0x82);
      Lcmd(0xca);
      for (i=0; i<4; i++)
        Ldat(ShotRegs[i] + '0');
      Mode = RunMode;
    }
  }
  else if (Mode == StopIntvlWait)
  {
    if (YesNo)
    {
      IntvlWait = 0;
      EEwrite(IntvlWaitLoc,0);
      ClearL2();
    }
    Mode = Idle;
  }
  else if ((Mode == DoneMode) || (Mode == ErrorMode))
    IdleModeReset();
}  

//==================== Menu Button Screen Routines ==========================//

void SetIntvlStartScreen(byte mode)
{
  Clear();
  HideCurs();
  SendStr(StartAutoStr,0x81);
  SendStr(NoStr,0xc6);
  YesNo = 0;
  Mode = mode;
}

void CloseStartPending()
{
  Clear();
  GetTime();
  ShowTime(0x82);
  Mode = RunMode;
  SetUpIntvlRun();  // Start the intervalometer
} 

void SetClockSetScreen(byte mode)
{
  Clear();
  SendStr(SetTimeStr,0x83);
  ShowCurs();
  LoadParmsetVals(0,2,12,1,0xc2,mode);
  ShowTime(DispBase);
}

void CloseClockSet()
{
  SetTime();
  GetTime();
  ShowTime(0x82);
}

void SetDateSetScreen(byte mode)
{
  Clear();
  SendStr(SetDateStr,0x84);
  ShowCurs();
  LoadParmsetVals(0,3,7,1,0xc1,mode);
  GetDate();
  ShowDate(DispBase);
  Lcmd(0xc1);
}

void CloseDateSet()
{
  SetDate();
  GetDate();
  ShowDate(0xc1);
}

void SetAlarmOnOffScreen(byte mode)
{
  Clear();
  SendStr(AlarmStr,0x83);
  Lcmd(0x8a);
  (AlmTime[0]) ? Ldat('n') : SendStr("ff",0);
  ShowAlmTime(0xc3);
  HideCurs();
  Mode = mode ;
} 

void SetAlarmTimeSet(byte mode)
{
  ClearL1();
  SendStr(SetAlarmStr,0x81);
  LoadParmsetVals(0,2,12,1,0xc3,mode);
  ShowCurs();
} 

void CloseAlmTimeSet()
{
  EEwrite(AlarmSetLoc,AlmTime[0]);
  EEwrite(AlarmHrs,AlmTime[1]);
  EEwrite(AlarmMins,AlmTime[2]);
  EEwrite(AlarmPM,AlmTime[3]);
  for (int i=0; i<3; i++)
    AlmScpd[i] = AlmTime[i + 1];
}

void SetShootTimeScreen(byte mode)
{
  SendStr(ShootStr,0x81);
  UpDownWork = EEPROM.read(ShootSecs);
  ShowShootTime();
  MaxVal = 99;
  MinVal = 1;
  Smudged = 0;
  Mode = mode;
}

void SetExpModeScreen(byte mode)
{
  HideCurs();
  Clear();
  SendStr(ChooseStr,0x81);
  ExpMode = EEPROM.read(ExpModeLoc);
  if (ExpMode == ExpNormal)
    SendStr(NormStr,0xc6);
  else if (ExpMode == ExpBulb)
    SendStr(BulbStr,0xc6);
  else
    SendStr(TimeExpStr,0xc6);
  Smudged = 0;
  Mode = mode;
}

void SetTimexTimeScreen(byte mode)
{
  Clear();    
  ShowCurs();                          //  Set up time exposure set rtn  
  SendStr(TimeExStr,0x82);  
  LoadTimeRegs(TimexFrac,TimexSetRegs);     //  Load time exposure time 
  //  from EEPROM
  Lcmd(0xc2);                            
  DispBase = 0xc2;
  ShowLongTime(TimexSetRegs);
  Lcmd(0xc2);
  LoadParmsetVals(0,7,9,0,0xc2,mode);
} 

void SetIntvlTimeScreen(byte mode)
{
  Clear();    
  ShowCurs();                               //  Set up interval set routine 
  SendStr(SetIntvlStr,0x82);
  LoadTimeRegs(IntvlFrac,IntvlSetRegs);     //  Load interval time 
  //  from EEPROM
  Lcmd(0xc2);
  ShowLongTime(IntvlSetRegs);
  Lcmd(0xc2);
  LoadParmsetVals(0,7,9,0,0xc2,mode);
}            

void SetMaxShotsScreen(byte mode)
{
  Clear();
  ShowCurs();
  SendStr(MaxShotStr,0x81);
  GetShots();
  LoadShots(Shots);  // parse to ShotRegs[0] (MSD) to ..[3] (LSD)
  Lcmd(0xc6);
  for (int i=0; i<4; i++)
    Ldat(ShotRegs[i] + '0');
  Lcmd(0xc6);
  LoadParmsetVals(0,3,9,0,0xc6,mode);
}  

void CloseMaxShots()
{
  int i,divisor,iwrk;

  divisor = 1000;
  iwrk = 0;
  for (i=0; i<4; i++)
  {
    iwrk += ShotRegs[i] * divisor;
    divisor /= 10;
  }
  EEwrite(ShotsHi,iwrk >> 8);
  EEwrite(ShotsLo,iwrk % 256);
}

void SetClearShotsScreen(byte mode)
{
  GetShots();
  LoadShots(ShotsTaken); // parse ShotsTaken to ShotRegs[0] (MSD) to ..[3] (LSD)
  Clear();
  Lcmd(0x81);
  for (int i=0; i<4; i++)
    Ldat(ShotRegs[i] + '0');
  SendStr(ShotsTakenStr,0);
  SendStr(ClearTokenStr,0xc3);
  SendStr(NoStr,0);
  YesNo = 0;
  HideCurs();
  Mode = mode;
}

void CloseClearShots()
{
  EEwrite(ShotsTakenHi,0);
  EEwrite(ShotsTakenLo,0);
  ShotsTaken = 0;
  for (int i=0; i<4; i++)
    ShotRegs[i] = 0;
}

void SetRunTimeScreen(byte mode)
{
  SendStr(RunTimeStr,0x82);
  ShowCurs();                               
  LoadTimeRegs(RunTimeFrac,RunTimeRegs);
  Lcmd(0xc2);
  ShowLongTime(RunTimeRegs);
  Lcmd(0xc2);
  LoadParmsetVals(0,7,9,0,0xc2,mode);
} 

void SetStartTimeScreen(byte mode)
{
  Clear();
  Get4DigTime(StartTimeRegs, StartTimeHrs, 3);
  Lcmd(0xc4);
  Show4DigTime(StartTimeRegs);
  SendStr(StartTimeStr,0x81);
  Lcmd(0xc4);
  ShowCurs();
  LoadParmsetVals(0,2,12,1,0xc4,mode);
}  

void SetStopTimeScreen(byte mode)
{
  Clear();
  Get4DigTime(StopTimeRegs, StopTimeHrs, 3);
  Lcmd(0xc4);
  Show4DigTime(StopTimeRegs);
  SendStr(StopTimeStr,0x81);
  Lcmd(0xc4);
  ShowCurs();
  LoadParmsetVals(0,2,12,1,0xc4,mode);
}    

void SetFocusScreen(byte mode)
{
  Clear();
  SendStr(FocusStr,0x81);
  UpDownWork = EEPROM.read(FocusSecs);
  ShowShootTime();
  Smudged = 0;
  MaxVal = 99;
  MinVal = 0;
  Mode = mode;
}  

void SetClrAutoScreen(byte mode)
{
  Clear();
  SendStr(ClearStr,0x81);
  YesNo = 0;
  SendStr(NoStr,0xc6);
  Mode = mode;
}  

void SetIntvlModeScreen(byte mode)
{
  Clear();             // set up interval mode select screen
  HideCurs();
  SendStr(AutoModeStr,0x81);
  Lcmd(0xc1);
  UpDownWork = EEPROM.read(IntvlModeLoc);
  ShowAutoMode();
  Smudged = 0;
  Mode = mode;
}   

void SetupBLBright(byte mode)
{
  Clear();
  SendStr(BLStr,0x81);
  ShowCurs();  
  BLWork = EEPROM.read(BLBright);
  Lcmd(0xc7);
  Ldat(BLWork + '0');
  BackSp();
  MaxVal = 4;
  MinVal = 0;
  Smudged = 0;
  Mode = mode;
}  

void SetStopAutoScreen(byte mode)
{
  SendStr(StopAutoStr,0x81);
  SendStr(NoStr,0xc6);
  YesNo = 0;
  Mode = mode;
} 

void SetupDayMapScreen(byte mode)
{
  Clear();
  HideCurs();
  SendStr(DayMapStr,0x82);
  GetDate();
  SendStr(DOWStr[DateRegs[0]],0xc4);
  if (DayMap & (1 << DateRegs[0]))
  {
    SendStr(YesStr,0xc8);
    YesNo = 1;
  }
  else
  {
    SendStr(NoStr,0xc8);
    YesNo = 0;
  }
  Mode = mode;
}

void ShowBusVoltage(byte mode)
{
  long wrk;

  //  Display the power bus voltage for 47k + 10k voltage divider, 
  //  5.0 v Aref
  HideCurs();
  Clear();
  SendStr(PwrBusStr,0x82);
  Lcmd(0xc4);
  wrk = (long) analogRead(PwrBus) * 50 * 57 / 1024; 
  if ((wrk / 1000) > 0)
  {
    Ldat(wrk / 1000 + '0');
    wrk = wrk % 1000;
  }
  Ldat(wrk / 100 + '0');
  wrk = wrk % 100;
  Ldat('.');      
  Ldat(wrk / 10 + '0');
  SendStr(VDCStr,0);
  Mode = mode;
}  

//===================== Menu Button Support Routines ===============//     

void LoadParmsetVals(byte DgPtr, byte MxDig, byte MxVal, 
byte MnVal, byte LCDaddr, byte ModeVal)
{
  DigPtr = DgPtr;
  MaxDig = MxDig;
  MaxVal = MxVal;
  MinVal = MnVal;
  DispBase = LCDaddr;
  Mode = ModeVal;
  Smudged = 0;
  Lcmd(LCDaddr);
}

void ClearAllIntervalometerSettings()
{
  int i;

  EEwrite(IntvlModeLoc,ShotLimited);
  IntvlMode = ShotLimited;

  for (i=0; i<8; i++)
  {
    IntvlSetRegs[i] = 0;
    IntvlRunRegs[i] = 0;
    RunTimeRegs[i] = 0; 
    EEwrite(IntvlFrac + i,0);
    EEwrite(RunTimeFrac + i,0);
  }

  IntvlSetTenths = 0;
  IntvlRunTenths = 0;
  RunTimeTenths = 0;

  EEwrite(ShotsHi,0);
  EEwrite(ShotsLo,0);
  Shots = 0;

  for (i=0; i<6; i++)
    EEwrite(StartTimeHrs + i,0);   // clear start and stop times
  EEwrite(StartTimeHrs,1);
  EEwrite(StopTimeHrs,1);

}   

void Get4DigTime(byte* reg, byte addr, byte count)
{
  int i;

  for (i=0; i<count; i++)
    *(reg + i) = EEPROM.read(addr + i);
}    

void Show4DigTime(byte* reg)
{
  byte ms,ls;

  bcd2ascii(*reg,&ms,&ls);
  Ldat(ms);
  Ldat(ls);
  Ldat(':');
  bcd2ascii(*(reg + 1),&ms,&ls);
  Ldat(ms);
  Ldat(ls);
  Ldat(' ');
  (*(reg + 2) == 0) ? Ldat('A') : Ldat('P');
  Ldat('M');
}  

void Save4DigTime(const byte* reg, byte addr, byte count)
{
  int i;

  for (i=0; i<count; i++)
    EEwrite(addr + i,*(reg + i));
}

//=======================  Select Button Routines ===================//

void doSelBtn()
{
  byte ms,ls;

  debounce(SelBtn);

  if (AlmSounding)
  {
    SetSnoozeAlm();
    return;
  }

  DigPtr++ ;
  if (DigPtr > MaxDig)
    DigPtr = 0;

  if ((Mode == ClockSet) || (Mode == AlarmSet))
  {
    if (!DigPtr)
    {
      Lcmd(DispBase);
      MinVal = 1;
      MaxVal = 12;
    }
    else if (DigPtr == 1)
    {
      Lcmd(DispBase + 3);
      MinVal = 0;
      MaxVal = 59;
    }
    else
      (Mode == ClockSet) ? Lcmd(DispBase + 9) : Lcmd(DispBase + 6);
  }
  else if (Mode == DateSetMode)
  {
    if (!DigPtr)            // day
    {
      Lcmd(0xc1);
      MaxVal = 7;
      MinVal = 1;
    }
    else if (DigPtr == 1)   // month
    {
      Lcmd(0xc5);
      MaxVal = 12;
      MinVal = 1;
    }
    else if (DigPtr == 2)   // date
    {
      Lcmd(0xc9);
      MaxVal = MaxDays[bcd2bin(DateRegs[1])];
      MinVal = 1;
    }
    else
    {
      Lcmd(0xcd);
      MaxVal = 99;
      MinVal = 0;
    }
  }
  else if (Mode == SetShots)
    Lcmd(DispBase + DigPtr);
  else if ((Mode == SetStartTime) || (Mode == SetStopTime) ||
    (Mode == SetStartTime))
  {
    if (!DigPtr)
    {
      Lcmd(0xc4);
      MaxVal = 12;
      MinVal = 1;
    }
    else if (DigPtr == 1)
    {
      Lcmd(0xc7);
      MaxVal = 59;
      MinVal = 0;
    }
    else if (DigPtr == 2)
      Lcmd(0xca);
  }
  else if (Mode == DayMapSetMode)
  {
    DateRegs[0]++ ;
    if (DateRegs[0] > 7)
      DateRegs[0] = 1;
    SendStr(DOWStr[DateRegs[0]],0xc4);
    if (DayMap & (1 << DateRegs[0]))
    {
      YesNo = 1;
      SendStr(YesStr,0xc8);
    }
    else
    {
      YesNo = 0;
      SendStr(NoStr,0xc8);
    }
  }
  else if ((Mode == SetTimeExp) || (Mode == SetInterval) ||
    (Mode == SetRunTime))
    IncLongDigPtr();
  else if ((Mode == DoneMode) || (Mode == ErrorMode))
    IdleModeReset();
}

void IncLongDigPtr()
{
  //  for long time display record

  if (!DigPtr)
    Lcmd(DispBase);
  else if ((DigPtr < 3) || (DigPtr == 4) || (DigPtr == 6))
    Forward();
  else if ((DigPtr == 3) || (DigPtr == 5) || (DigPtr == 7))
  {
    Forward();
    Forward();
  }

  if ((DigPtr < 3) || (DigPtr == 4) || (DigPtr > 5))
  {
    MaxVal = 9;
    MinVal = 0;
  }
  else if ((DigPtr == 3) || (DigPtr == 5))
  {
    MaxVal = 5;
    MinVal = 0;
  }

  if (((Mode == SetInterval) 
    || (Mode == SetRunTime)) 
    && (DigPtr == 7))
  {
    //  Block cursor from tenths digit
    DigPtr = 0;
    Lcmd(DispBase);
  }
} 

//===================== Common Up/Down Button Routines ==========//

void ClkSetUpDown(byte updn)
{
  if(!DigPtr)
    UpdateTimeField(&Hrs,DispBase,updn);
  else if (DigPtr == 1)
    UpdateTimeField(&Mins,DispBase + 3,updn);
  else
    TogglePMflag(&PMFlag,DispBase + 9);
}

void ToggleDate(byte incdec)
{
  byte x;

  x = bcd2bin(DateRegs[DigPtr]);
  if (incdec)
  {
    x++ ;
    if (x > MaxVal)
      x = MinVal;
  }
  else
    (x == MinVal) ? x = MaxVal : x-- ;
  DateRegs[DigPtr] = bin2bcd(x);
  ShowDate(0xc1);
  if (!DigPtr)
    Lcmd(0xc1);
  else if (DigPtr == 1)
    Lcmd(0xc5);
  else if (DigPtr == 2)
    Lcmd(0xc9);
  else
    Lcmd(0xcd);
}  

void UpdTimeFieldWrapper(byte* reg, byte addr, byte updn)
{
  if (!DigPtr)
    UpdateTimeField(reg,addr,updn);
  else if (DigPtr == 1)
    UpdateTimeField(reg+1,addr+3,updn);
  else
    TogglePMflag(reg+2,addr+6);
}

void ToggleAlarm()
{
  if (AlmTime[0])
  {
    AlmTime[0] = 0;
    SendStr("ff",0x8a);
    ClearAlmIcon();
  }
  else
  {
    AlmTime[0] = 1;
    SendStr("n ",0x8a);
    ShowAlmIcon();
  }
  EEwrite(AlarmSetLoc,AlmTime[0]);   
}

void CycleExpMode()
{
  if (ExpMode == ExpNormal)
  {
    ExpMode = ExpBulb;
    SendStr(BulbStr,0xc6);
  }
  else if (ExpMode == ExpBulb)
  {
    ExpMode = ExpTimeEx;
    SendStr(TimeExpStr,0xc6);
  }
  else    //  ExpTimeEx
  {
    ExpMode = ExpNormal;
    SendStr(NormStr,0xc6);
  }
}

void ToggleIntvlYesno(byte addr)
{
  if (YesNo)
  {
    YesNo = 0;
    SendStr(NoStr,addr);
  }
  else
  {
    YesNo = 1;
    SendStr(YesStr,addr);
  }
}  

void AutoModeSel()
{
  UpDownWork++ ;
  if (UpDownWork > Timed)
    UpDownWork = ShotLimited;
  ClearL2();
  Lcmd(0xc1);
  ShowAutoMode();
}

void UpdShotsDig(int incdec)
{
  if (incdec)
  {
    ShotRegs[DigPtr]++ ;
    if (ShotRegs[DigPtr] > MaxVal)
      ShotRegs[DigPtr] = MinVal;
  }
  else
  {
    if (ShotRegs[DigPtr] == MinVal)
      ShotRegs[DigPtr] = MaxVal;
    else
      ShotRegs[DigPtr]-- ;
  }
  Ldat(ShotRegs[DigPtr] + '0');
  BackSp();
}  

void BLChange(byte incdec)
{
  if (incdec)
  {
    BLWork++ ;
    if (BLWork > MaxVal)
      BLWork = MinVal;
  }
  else
  {
    if (BLWork == MinVal)
      BLWork = MaxVal;
    else
      BLWork-- ;
  }
  Ldat(BLWork + '0');
  BackSp();
  analogWrite(BLpin,BLVals[BLWork]);
}  

//========================  Up Button Routines ==================//

void doUpBtn()
{
  byte x;

  debounce(UpBtn);

  if (AlmSounding)
  {
    SetSnoozeAlm();
    return;
  }

  Smudged = 1;

  if (Mode == ClockSet)
    ClkSetUpDown(1);
  else if (Mode == Alarm)
    ToggleAlarm();
  else if (Mode == AlarmSet)
    UpdTimeFieldWrapper(&AlmTime[1],0xc3,1);
  else if (Mode == DateSetMode)
    ToggleDate(1);
  else if ((Mode == StopPending) 
        || (Mode == StartPending) 
        || (Mode == StopIntvlWait))    
           ToggleIntvlYesno(0xc6);
  else if ((Mode == Focus) 
        || (Mode == Shoot))
  {
    UpDownWork++ ;
    if (UpDownWork > MaxVal)
      UpDownWork = MinVal;
    ShowShootTime();
  }
  else if (Mode == SetExpMode)
    CycleExpMode();
  else if (Mode == SetTimeExp)
    ChangeLongDigit(TimexSetRegs, 1);
  else if (Mode == ClrAuto)
  {
    YesNo = 1 - YesNo;
    (YesNo) ? SendStr(YesStr,0xc6) : SendStr(NoStr,0xc6);
  }
  else if (Mode == SetInterval)
    ChangeLongDigit(IntvlSetRegs, 1);
  else if (Mode == ClearShots)
  {
    YesNo = 1 - YesNo;
    (YesNo) ? SendStr(YesStr,0xca) : SendStr(NoStr,0xca);
  }
  else if (Mode == SetAutoMode)
    AutoModeSel();
  else if (Mode == SetShots)
    UpdShotsDig(1);
  else if (Mode == SetRunTime)
    ChangeLongDigit(RunTimeRegs, 1);
  else if (Mode == SetStartTime)
    UpdTimeFieldWrapper(&StartTimeRegs[0],0xc4,1);   
  else if (Mode == SetStopTime)
    UpdTimeFieldWrapper(&StopTimeRegs[0],0xc4,1);
  else if (Mode == DayMapSetMode)
  {
    ToggleIntvlYesno(0xc8);
    (YesNo) ? DayMap |= (1 << DateRegs[0]) : DayMap &= ~(1 << DateRegs[0]); 
  }    
  else if (Mode == BLSet)
    BLChange(1);
  else if ((Mode == DoneMode) || (Mode == ErrorMode))
    IdleModeReset();
}

//========================== Down Button Routines ================//

void doDownBtn()
{
  byte x,y,z;

  debounce(DownBtn);

  if (AlmSounding)
  {
    SetSnoozeAlm();
    return;
  }

  Smudged = 1;

  if (Mode == ClockSet)
    ClkSetUpDown(0);      
  else if (Mode == Alarm)
    ToggleAlarm();
  else if (Mode == AlarmSet)
    UpdTimeFieldWrapper(&AlmTime[1],0xc3,0);
  else if (Mode == DateSetMode)
    ToggleDate(0);
  else if ((Mode == StopPending) || (Mode == StartPending) ||
    (Mode == StopIntvlWait))   
    ToggleIntvlYesno(0xc6);
  else if ((Mode == Focus) || (Mode == Shoot))
  {
    (UpDownWork == MinVal) ? UpDownWork = MaxVal : UpDownWork-- ;
    ShowShootTime();
  }
  else if (Mode == SetExpMode)
    CycleExpMode();
  else if (Mode == SetTimeExp)
    ChangeLongDigit(TimexSetRegs, 0);
  else if (Mode == ClrAuto)
  {
    YesNo = 1 - YesNo;
    (YesNo) ? SendStr(YesStr,0xc6) : SendStr(NoStr,0xc6);
  }
  else if (Mode == SetInterval)
    ChangeLongDigit(IntvlSetRegs,0);
  else if (Mode == ClearShots)
  {
    YesNo = 1 - YesNo;
    (YesNo) ? SendStr(YesStr,0xca) : SendStr(NoStr,0xca);
  }
  else if (Mode == SetAutoMode)
    AutoModeSel();
  else if (Mode == SetShots)
    UpdShotsDig(0);
  else if (Mode == SetRunTime)
    ChangeLongDigit(RunTimeRegs, 0);
  else if (Mode == SetStartTime)
    UpdTimeFieldWrapper(&StartTimeRegs[0],0xc4,0);   
  else if (Mode == SetStopTime)
    UpdTimeFieldWrapper(&StopTimeRegs[0],0xc4,0); 
  else if (Mode == DayMapSetMode)
  {
    ToggleIntvlYesno(0xc8);
    (YesNo) ? DayMap |= (1 << DateRegs[0]) : DayMap &= ~(1 << DateRegs[0]); 
  }    
  else if (Mode == BLSet)
    BLChange(0);
  else if ((Mode == DoneMode) || (Mode == ErrorMode))
    IdleModeReset();
}

void UpdateTimeField(byte* bcdreg, byte fieldaddr, byte incdec)
{
  byte wrk,ms,ls;

  wrk = bcd2bin(*bcdreg);

  if (incdec)
  {
    wrk++ ;    
    if (wrk > MaxVal)
      wrk = MinVal;
  }
  else
    (wrk == MinVal) ? wrk = MaxVal : wrk-- ;
  Lcmd(fieldaddr);
  bin2ascii(wrk, &ms, &ls);
  Ldat(ms);
  Ldat(ls);
  Lcmd(fieldaddr);
  *bcdreg = bin2bcd(wrk);
}  

void TogglePMflag(byte* flag, byte addr)
{
  Lcmd(addr);
  if (*flag)
  {
    *flag = 0;
    Ldat('A');
  }
  else
  {
    *flag = 1;
    Ldat('P');
  }
  Ldat('M');
  Lcmd(addr);
}

void ShowAlmTime(byte addr)
{
  byte ms,ls;

  Lcmd(addr);  
  bcd2ascii(AlmTime[1],&ms,&ls);
  Ldat(ms);
  Ldat(ls);
  Ldat(':');
  bcd2ascii(AlmTime[2],&ms,&ls);
  Ldat(ms);
  Ldat(ls);
  SendStr("  M",0);
  Lcmd(addr + 6);
  (AlmTime[3]) ? Ldat('P'): Ldat('A');
  Lcmd(addr);
} 

void ShowLongTime(byte* reg)
{
  int i;

  for (i=7;  i>4; i--)
    Ldat(*(reg + i) + '0');
  Ldat(':');
  for (i=4; i>2; i--)
    Ldat(*(reg + i) + '0'); 
  Ldat(':');
  for (i=2; i>0; i--)
    Ldat(*(reg + i) + '0'); 
  Ldat('.');
  Ldat(*reg + '0');
}    

void ChangeLongDigit(byte* reg, byte incdec)
{
  byte index;

  index = MaxDig - DigPtr;

  if (incdec)
  {
    *(reg + index) += 1;
    if (*(reg + index) > MaxVal)
      *(reg + index) = MinVal;
  }
  else
    if (*(reg + index) == MinVal)
      *(reg + index) = MaxVal ;
    else
      *(reg + index) -= 1;
  Ldat(*(reg + index) + '0');
  BackSp();
}    

//=======================  Service Routines ======================//

void GetTime()
{
  Secs = get1302data(ReadSecs);
  Mins = get1302data(ReadMins);
  Hrs = get1302data(ReadHrs);
  ((Hrs & B00100000) > 0) ? PMFlag = 1 : PMFlag = 0;
  Hrs &= B00011111;
}

void ShowTime(byte addr)
{
  byte ms,ls;

  bcd2ascii(Hrs,&ms,&ls);
  Lcmd(addr);
  if ((Mode != ClockSet) && (ms == '0'))
    Ldat(' ');
  else
    Ldat(ms);
  Ldat(ls);
  Ldat(':');
  bcd2ascii(Mins,&ms,&ls);
  Ldat(ms);
  Ldat(ls);
  Ldat(':');
  bcd2ascii(Secs,&ms,&ls);
  Ldat(ms);
  Ldat(ls);
  Ldat(' ');
  (PMFlag) ? Ldat('P') : Ldat('A');
  Ldat('M');
  Lcmd(addr);
}   

void SetTime()
{
  //  Write time to DS1302

  byte temp;

  (PMFlag) ? temp = Hrs | B10100000 : temp = Hrs | B10000000;
  send1302cmd(WriteHrs,temp);
  send1302cmd(WriteMins,Mins);
  send1302cmd(WriteSecs,0);
}

void GetDate()
{  
  DateRegs[0] = get1302data(ReadDay);  // 1-7, Sun = 1
  DateRegs[1] = get1302data(ReadMonth);
  DateRegs[2] = get1302data(ReadDate);
  DateRegs[3] = get1302data(ReadYear);
}

void SetDate()
{
  //  Write date to DS1302

  byte temp;

  send1302cmd(WriteDay,DateRegs[0]);
  send1302cmd(WriteMonth,DateRegs[1]);
  send1302cmd(WriteDate,DateRegs[2]);
  send1302cmd(WriteYear,DateRegs[3]);
  temp = get1302data(ReadSecs);    // lock in the changes
  send1302cmd(WriteSecs,temp);
}

void ShowDate(byte addr)
{
  byte ms,ls;

  SendStr(DOWStr[DateRegs[0]],addr);
  Ldat(' ');
  SendStr(MonthStr[bcd2bin(DateRegs[1])],0);
  Ldat(' ');
  bcd2ascii(DateRegs[2],&ms,&ls);
  Ldat(ms);
  Ldat(ls);
  SendStr(", ",0);
  bcd2ascii(DateRegs[3],&ms,&ls);
  Ldat(ms);
  Ldat(ls);
}   

void EEwrite(byte addr, byte data)
{
  EEPROM.write(addr,data);
  delay(10);
}  

void bcd2ascii(const byte val, byte* ms, byte* ls)
{
  *ms = (val >> 4) + '0';
  *ls = (val & B00001111) + '0';
}

void bin2ascii(const byte val, byte* ms, byte* ls)
{
  *ms = val / 10 + '0';
  *ls = val % 10 + '0';
}  

byte bcd2bin(const byte val)
{
  return (((val >> 4) * 10) + (val & B00001111));
}

byte bin2bcd(const byte val)
{
  return ((val / 10 * 16) + (val % 10));
}  

// ====================  8-byte Time Routines  ========= //
void LoadTimeRegs(byte eepromaddr, byte* targaddr)
{
  // Loads 8 bytes from EEPROM to RAM
  int i;

  for (i=0; i < 8; i++)
    *(targaddr + i) = EEPROM.read(eepromaddr + i);
}

void SaveTimeRegs(byte eepromaddr, byte* sourceaddr)
{
  //  Stores 8 RAM bytes from RAM to EEPROM

  for (int i=0; i < 8; i++)
    EEwrite(eepromaddr + i, *(sourceaddr + i));
}

boolean DecTimeRegs(byte* sourcereg, long* tenths_reg)
{
  //  Decrements 9 sequential regs @sourceaddr and calculates 
  //  tenths remaining to @tenths_reg.

  //  Regs are in the format Tenths, Sx1, Sx10, Mx1, Mx10, Hx1, Hx10, 
  //  Hx100, dummy

  //  Returns 0 if decremented value not zero, 1 if zero.
  //  Will not decrement below zero.

  //  Tested and working Jul 13, 07 tjl

  boolean flag = 0;
  int i;

  //  Check if regs are all zero
  for (i=0; i<8; i++)
    if (*(sourcereg + i) != 0)
    {
      flag = 1;
      break;
    }
  if (!flag)
    return (1);
  else
  {  
    for (i=0; i < 9; i++)
    {
      *(sourcereg + i) -= 1 ;
      if(*(sourcereg + i) > 9)
        *(sourcereg + i) = TimeMax[i];
      else
        break;
    }
    CalcTenthsRemaining(sourcereg,tenths_reg);
    return (*tenths_reg == 0);
  }     
}

void CalcTenthsRemaining(byte* source, long* tenths_reg)
{    
  *tenths_reg = *(source + 7) * 3600L * 100 * 10 +     // Hrs x 100
  *(source + 6) * 3600L * 10 * 10 +      // Hrs x 10
  *(source + 5) * 3600L * 10 +           // Hrs
  *(source + 4) * 60 * 10 * 10 +         // Min x 10
  *(source + 3) * 60 * 10 +              // Min
  *(source + 2) * 10 * 10 +              // Sec x 10
  *(source + 1) * 10 +                   // Sec
  *source;                               // Tenths
} 

//================  Shoot and External Trigger routines  ==============//

void TakeAPicture()
{
  //  In setup mode, the Shoot button provides an immediate escape
  //  to Idle mode.  
  if (digitalRead(ShootSw) && ((Mode != Idle) && (Mode != RunMode)))
  {
    while(digitalRead(ShootSw))
      delay(5);
    delay(5);
    Mode = Idle;
    Clear();
    GetTime();
    HideCurs();
    ShowTime(0x82);
    return;
  }

  IncShotsTaken();

  //  Converted to an if-then-else structure, this routine
  //  acutally compiled 4 bytes larger than a switch structure

  ExpMode = EEPROM.read(ExpModeLoc);
  switch (ExpMode)
  {
  case ExpNormal:
    {
      NormalExposure();
      break;
    }
  case ExpBulb:
    {
      BulbExposure();
      break;
    }
  default:
    TimeExposure();
  }
}

void NormalExposure()
{
  byte eFoc, eShoot;

  eFoc = EEPROM.read(FocusSecs);
  eShoot = EEPROM.read(ShootSecs);
  PORTB |= B00010000;                  //  FocusPin high
  delay(eFoc * 100);
  PORTB |= B00100000;                  //  ShootPin high
  delay(eShoot * 100);
  PORTB &= B11001111;                  //  FocusPin & ShootPin low
  while(digitalRead(ShootSw) || !digitalRead(ExtTrig))
    delay(5);
  delay(5);
} 

void BulbExposure()
{
  byte eFoc;

  eFoc = EEPROM.read(FocusSecs);
  PORTB |= B00010000;                  //  FocusPin high
  delay(eFoc * 100);
  PORTB |= B00100000;                  //  ShootPin high
  while(digitalRead(ShootSw) || !digitalRead(ExtTrig))
    delay(5);
  delay(5);
  PORTB &= B11001111;                  //  FocusPin & ShootPin low
} 

void TimeExposure()
{
  byte FocSecs;

  FocSecs = EEPROM.read(FocusSecs);
  LoadTimeRegs(TimexFrac,TimexRunRegs);
  CalcTenthsRemaining(TimexRunRegs,&TimexRunTenths);

  PORTB |= B00010000;                  //  FocusPin high
  delay(FocSecs * 100);
  PORTB |= B00100000;                  //  ShootPin high
  while(!DecTimeRegs(TimexRunRegs,&TimexRunTenths))
    delay(100);
  PORTB &= B11001111;                  //  FocusPin & ShootPin low

  while(digitalRead(ShootSw) || !digitalRead(ExtTrig))
    delay(5);
  delay(5);
} 

//======================== Alarm Clock Routines =====================//

void Beep()
{
  int i,j;

  for (i=0; i<4; i++)
  {
    for (j=0; j<400; j++)
    {

      PORTB |= B00001000;                    //  AlarmPin pin high
      delayMicroseconds(150);
      PORTB &= B11110111;                    //  AlarmPin pin low
      delayMicroseconds(150); 
    }
    TopScan();
    TopScan();
    TopScan();
  }
}

void Inc4DigBCDTime(byte *reg, byte incmins)
{
  //  Adds incmins to the pointed bcd register set (hrs, mins, PMflag) and
  //  places the result back into the pointed register set

  byte hh,mm;
  byte flag;

  hh = bcd2bin(*reg);
  mm = bcd2bin(*(reg+1));
  mm += incmins ;
  if (mm > 59)
  {
    mm = 0;
    hh++ ;
    if (hh == 12)
      *(reg + 2) = 1 - *(reg + 2);     //  Toggle AM/PM flag
    else if (hh > 12)
      hh = 1;
  }
  *reg = bin2bcd(hh);
  *(reg + 1) = bin2bcd(mm);
} 

//======================= Intervalometer Routines =======================//

void SetUpIntvlRun()
{
  byte x=0;

  x = CheckIntvlStartErrs();
  if (x)
    return;

  if (IntvlMode != Timed)            //  Clear shots taken
  {
    for (int i=0; i<4; i++)
      ShotRegs[i] = 0;
    ShotsTaken = 0;
    EEwrite(ShotsTakenHi,0);
    EEwrite(ShotsTakenLo,0);
  }

  Mode = RunMode;           //  Start intervalometer run
  ClearL2();
  TakeAPicture();
}

void ResetIntvl()
{
  TakeAPicture();
  GetInterval();
}      

void GetInterval()
{
  //  Returns effective interval, in seconds, in IntvlRunTenths

  LoadTimeRegs(IntvlFrac,IntvlRunRegs);
  CalcTenthsRemaining(IntvlRunRegs,&IntvlRunTenths);
  IntvlRunTenths /= 10;
  IntvlRunTenths++ ;
}

void DisplayCountdown()
{
  int i;
  long wrk;
  long divisor = 1000000;

  if (Mode != RunMode)
    return;

  wrk = IntvlRunTenths;
  Lcmd(0xc2);
  for (i=0; i<7; i++)
  {
    Ldat(wrk / divisor + '0');
    wrk %= divisor;
    divisor /= 10;
  }
}

byte CheckIntvlStartErrs()
{
  //  Init intervalometer vars and check for startup errors

    byte errcode = 0;

  GetInterval();  // returns interval in seconds
  Shots = EEPROM.read(ShotsHi) * 256 + EEPROM.read(ShotsLo);
  IntvlMode = EEPROM.read(IntvlModeLoc); 
  ExpMode = EEPROM.read(ExpModeLoc); 
  GetShots();  
  if (ExpMode == ExpBulb)
  {
    ShowIntvlSetupErrMsg(BulbModeErrStr1,0x81,
    BulbModeErrStr2,0xc1);
    errcode++ ;
  }
  else if (Shots == 0)
  {
    ShowIntvlSetupErrMsg(MaxShotsErrStr1,0x83,
    MaxShotsErrStr2,0xc2);
    errcode++ ;
  }
  else if (IntvlMode == TimeLimited)
  {
    LoadTimeRegs(RunTimeFrac,RunTimeRegs);
    CalcTenthsRemaining(RunTimeRegs,&RunTimeTenths);
    RunTimeTenths /= 10;
    if (RunTimeTenths == 0)
    {  
      ShowIntvlSetupErrMsg(DurErrStr1,0x82,
      DurErrStr2,0xc2);
      errcode++ ;
    }
  }
  else if ((IntvlMode == Timed) && (ShotsTaken >= Shots))
  {
    ShowIntvlSetupErrMsg(ShotsTakenErrStr,0x81,
    MaxShotsErrStr1,0xc3);
    errcode++ ;
  }
  return (errcode);
} 

void ShowIntvlSetupErrMsg(const char* s1, byte addr1,
const char* s2, byte addr2)
{
  Clear();
  Mode = ErrorMode;
  SendStr(s1,addr1);
  SendStr(s2,addr2);
  Beep();  
  delay(1000);
}                           

void IncShotsTaken()
{
  GetShots();
  ShotsTaken++ ;
  LoadShots(ShotsTaken);
  EEwrite(ShotsTakenHi,ShotsTaken / 256);
  EEwrite(ShotsTakenLo,ShotsTaken % 256);

  if (Mode == RunMode)
  {
    ShowShots();
    if (ShotsTaken == Shots)
    {
      SetDoneMode();
    }
  }
  if (Mode == Idle)
    ShowShots();
}

void GetShots()
{
  Shots = EEPROM.read(ShotsHi) * 256 + EEPROM.read(ShotsLo);
  ShotsTaken = EEPROM.read(ShotsTakenHi) * 256 + EEPROM.read(ShotsTakenLo);
} 

void LoadShots(int shots)
{
  //  Loads shots into ShotRegs[0] -> ..[3]
  int i,wrk;
  int divisor = 1000;

  wrk = shots;
  for (i=0; i<4; i++)
  {
    ShotRegs[i] = wrk / divisor;
    wrk %= divisor;
    divisor /= 10;
  }
}  

void SetDoneMode()
{
  Mode = DoneMode;
  DonePending = 1;
  ClearL2();
  SendStr(DoneStr,0xc6);
  if (IntvlMode == Timed)
    IntvlWait = 1;
} 

void ShowShots()
{
  Lcmd(0xca);
  for (int i=0; i<4; i++)
    Ldat(ShotRegs[i] + '0');
}  

// ====================  LCD Functions  ==========================//

void Clear()
{
  Lcmd(0x01);
  ShowIcons();
}

void ClearL1() 
{
  //  Clear LCD line 1
  Lcmd(0x80);
  for (int i=0; i<15; i++)
    Ldat(' ');
  ShowIcons();  
}

void ClearL2() 
{
  //  Clear LCD line 2
  Lcmd(0xc1);
  for (int i=0; i<14; i++)
    Ldat(' ');
  ShowIcons();  
}

void ShowIcons()
{
  (LowBattFlag) ? ShowLBIcon() : ClearLBIcon();
  (AlmTime[0]) ? ShowAlmIcon() : ClearAlmIcon();    
  (IntvlWait) ?  ShowIntvlIcon() : ClearIntvlIcon(); 
}  

void ShowLBIcon()
{
  Lcmd(0x8f);
  Ldat(1);
}

void ClearLBIcon()
{  
  Lcmd(0x8f);
  Ldat(' ');
} 

void ShowAlmIcon()
{
  Lcmd(0xcf);
  Ldat(0);
}

void ClearAlmIcon()
{
  Lcmd(0xcf);
  Ldat(' ');
}

void ShowIntvlIcon()
{
  Lcmd(0xc0);
  Ldat(2);
}

void ClearIntvlIcon()
{
  Lcmd(0xc0);
  Ldat(' ');
}  

void Forward()
{
  Lcmd(0x14);
}

void BackSp()
{ 
  Lcmd(0x10);
}  

void HideCurs()
{
  Lcmd(0x0c);
}

void ShowCurs()
{
  Lcmd(0x0e);
}

// ===================  LCD Driver Routines  ========================//

void SendStr(const char* str, const byte addr)
{
  //  Send string at addr, if addr <> 0, or cursor position  if addr == 0
  int i = 0;

  if (addr != 0)
    Lcmd(addr);
  while (str[i] != 0)
    Ldat(str[i++]);
} 

void Lcmd(byte Cmd)
{
  //  Sends a byte to LCD in command mode

  PORTC &= B11101111;                        // RS low
  Lout(Cmd);  
}  

void Ldat(byte Dat)
{
  //  Sends a byte to LCD in data mode

  PORTC |= B00010000;                        // RS high
  Lout(Dat);
}  

void Lout(byte Dat)
{
  //  Sends Dat to LCD.  Calling routine sets RS to Command (Low) or 
  //  Data (High) mode

  clock2LCD(Dat >> 4);           //  Place high nibble on low bits 
  clock2LCD(Dat & B00001111);    // Send low nibble
}

void clock2LCD(byte nibble)
{
  //  Clock high nibble of temp to LCD

  PORTC &= B11110000;
  PORTC |= nibble;
  PORTB |= B00000100;             //  Strobe data using E
  delay(1);
  PORTB &= B11111011;
  delay(1);
}  

// ===================  DS1302 Driver Routines ==================//

void send1302cmd(byte cmd1, byte cmd2)
{	
  PORTB |= B00000001;                          // Set RST1302 high
  delayMicroseconds(2);
  shiftOut(DAT1302,CLK1302,LSBFIRST,cmd1); 
  delayMicroseconds(2);                        //  This delay might not 
  //  be needed
  shiftOut(DAT1302,CLK1302,LSBFIRST,cmd2);
  PORTB &= B11111110;                          // Set RST1302 low
}

byte get1302data(byte cmd)
{
  byte dat;

  PORTB |= B00000001;                          // RST1302 high
  shiftOut(DAT1302,CLK1302,LSBFIRST,cmd);
  dat = shiftin();
  //
  PORTB &= B11111110;                          //  RST1302 low
  return (dat);
}  

byte shiftin()
{
  //  Shifts in a byte from the DS1302.  

  //  This routine is called immediately after a shiftout.  The first bit is 
  //  present on DAT1302 at call, so only 7 additional clock pulses are 
  //  required.

  //  Restores DAT1302 to OUTPUT prior to return.

  byte dat = 0;
  int i = 0;

  DDRD &= B01111111;                // Set DAT1302 as input

  for(i=0; i < 7; i++)
  {  
    if(PIND & B10000000)            // DAT1302 high?
      dat |= B10000000;
    dat >>= 1;
    PORTD |= B01000000;             // Strobe in next data bit using CLK1302
    delay(1);
    PORTD &= B10111111;
    delay(1);
  }

  DDRD |= B10000000;                // Restore DAT1302 as output

  return (dat); 
}  

//==========================  Debug Routines ================= ==========//
/*
void bp()
 {
 //  Stops execution until the ext trig button is pressed
 
 while(PIND & B00001000)      // ExtTrig high
 delay(5);
 while(!(PIND & B00001000))   // ExtTrig low
 delay(5);
 delay(5);
 }  
 
 void bps1(const char* str)
 {
 //  Displays str on line 1, waits for ext trig button press
 
 ClearL1();
 SendStr(str,0x81);
 bp();
 }  
 
 void bps2(const char* str)
 {
 //  Displays str on line 2, waits for ext trig button press
 
 ClearL2();
 SendStr(str,0xc1);
 bp();
 }
 */







